@@ -1,9 +1,9 @@
 /*
- * "$Id: config.h 327 2007-12-06 06:07:50Z mike $"
+ * "$Id: config.h 387 2009-04-18 17:05:52Z mike $"
  *
  * Configuration file for Mini-XML, a small XML-like file parsing library.
  *
- * Copyright 2003-2007 by Michael Sweet.
+ * Copyright 2003-2009 by Michael Sweet.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,9 +22,26 @@
  * when many of the affected functions are otherwise completely secure.
  * The _CRT_SECURE_NO_DEPRECATE definition ensures that we won't get
  * warnings from their use...
+ *
+ * Then Microsoft decided that they should ignore this in VC2008 and use
+ * yet another define (_CRT_SECURE_NO_WARNINGS) instead.  Bastards.
  */
 
 #define _CRT_SECURE_NO_DEPRECATE
+#define _CRT_SECURE_NO_WARNINGS
+
+
+/*
+ * Include necessary headers...
+ */
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdarg.h>
+#include <ctype.h>
+#include <io.h>
+
 
 /*
  * Microsoft also renames the POSIX functions to _name, and introduces
@@ -45,22 +62,11 @@
 #define write		_write
 
 
-/*
- * Include necessary headers...
- */
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdarg.h>
-#include <ctype.h>
-
-
 /*
  * Version number...
  */
 
-#define MXML_VERSION "Mini-XML v2.4.1"
+#define MXML_VERSION "Mini-XML v2.6"
 
 
 /*
@@ -70,6 +76,13 @@
 #define inline _inline
 
 
+/*
+ * Long long support...
+ */
+
+#define HAVE_LONG_LONG 1
+
+
 /*
  * Do we have the snprintf() and vsnprintf() functions?
  */
@@ -108,5 +121,5 @@ extern int	_mxml_vsnprintf(char *, size_t, const char *, va_list);
 #  endif /* !HAVE_VSNPRINTF */
 
 /*
- * End of "$Id: config.h 327 2007-12-06 06:07:50Z mike $".
+ * End of "$Id: config.h 387 2009-04-18 17:05:52Z mike $".
  */
